@@ -1,6 +1,6 @@
-from .auth import NetatmoOAuth2
+from .auth import AbstractAsyncAuth, NetatmoOAuth2
 from .helpers import _BASE_URL
-from .weather_station import WeatherStationData
+from .weather_station import AsyncWeatherStationData, WeatherStationData
 
 _GETHOMECOACHDATA_REQ = _BASE_URL + "api/gethomecoachsdata"
 
@@ -15,8 +15,19 @@ class HomeCoachData(WeatherStationData):
 
         Arguments:
             auth {NetatmoOAuth2} -- Authentication information with a valid access token
+        """
+        super().__init__(auth, url_req=_GETHOMECOACHDATA_REQ)
 
-        Raises:
-            NoDevice: No devices found.
+
+class AsyncHomeCoachData(AsyncWeatherStationData):
+    """
+    Class of Netatmo Home Couch devices (stations and modules)
+    """
+
+    def __init__(self, auth: AbstractAsyncAuth) -> None:
+        """Initialize self.
+
+        Arguments:
+            auth {AbstractAsyncAuth} -- Authentication information with a valid access token
         """
         super().__init__(auth, url_req=_GETHOMECOACHDATA_REQ)
