@@ -1,8 +1,10 @@
+"""Support for Netatmo energy devices (relays, thermostats and valves)."""
 import logging
+from abc import ABC
 from collections import defaultdict
 from typing import Any, Dict, Optional
 
-from .auth import NetatmoOAuth2
+from .auth import AbstractAsyncAuth, NetatmoOAuth2
 from .exceptions import InvalidRoom, NoDevice, NoSchedule
 from .helpers import _BASE_URL
 
@@ -16,36 +18,20 @@ _GETROOMMEASURE_REQ = _BASE_URL + "api/getroommeasure"
 _SWITCHHOMESCHEDULE_REQ = _BASE_URL + "api/switchhomeschedule"
 
 
-class HomeData:
-    """
-    Class of Netatmo energy devices (relays, thermostat modules and valves)
-    """
+class AbstractHomeData(ABC):
+    """Abstract class of Netatmo energy devices."""
 
-    def __init__(self, auth: NetatmoOAuth2) -> None:
-        """Initialize self.
+    raw_data: Dict = defaultdict(dict)
+    homes: Dict = defaultdict(dict)
+    modules: Dict = defaultdict(dict)
+    rooms: Dict = defaultdict(dict)
+    schedules: Dict = defaultdict(dict)
+    zones: Dict = defaultdict(dict)
+    setpoint_duration: Dict = defaultdict(dict)
 
-        Arguments:
-            auth {NetatmoOAuth2} -- Authentication information with a valid access token
-
-        Raises:
-            NoDevice: No devices found.
-        """
-        self.auth = auth
-        resp = self.auth.post_request(url=_GETHOMESDATA_REQ)
-        if resp is None or "body" not in resp:
-            raise NoDevice("No thermostat data returned by Netatmo server")
-
-        self.raw_data = resp["body"].get("homes")
-        if not self.raw_data:
-            raise NoDevice("No thermostat data available")
-
-        self.homes: Dict = {d["id"]: d for d in self.raw_data}
-
-        self.modules: Dict = defaultdict(dict)
-        self.rooms: Dict = defaultdict(dict)
-        self.schedules: Dict = defaultdict(dict)
-        self.zones: Dict = defaultdict(dict)
-        self.setpoint_duration: Dict = defaultdict(dict)
+    def process(self) -> None:
+        """Process data from API."""
+        self.homes = {d["id"]: d for d in self.raw_data}
 
         for item in self.raw_data:
             home_id = item.get("id")
@@ -86,22 +72,6 @@ class HomeData:
 
         return {}
 
-    def switch_home_schedule(self, home_id: str, schedule_id: str) -> Any:
-        """Switch the schedule for a give home ID."""
-        schedules = {
-            self.schedules[home_id][s]["name"]: self.schedules[home_id][s]["id"]
-            for s in self.schedules.get(home_id, {})
-        }
-        if schedule_id not in list(schedules.values()):
-            raise NoSchedule("%s is not a valid schedule id" % schedule_id)
-
-        post_params = {
-            "home_id": home_id,
-            "schedule_id": schedule_id,
-        }
-        resp = self.auth.post_request(url=_SWITCHHOMESCHEDULE_REQ, params=post_params)
-        LOG.debug("Response: %s", resp)
-
     def get_hg_temp(self, home_id: str) -> Optional[float]:
         """Return frost guard temperature value."""
         return self._get_selected_schedule(home_id).get("hg_temp")
@@ -118,30 +88,95 @@ class HomeData:
 
         return None
 
+    def is_valid_schedule(self, home_id: str, schedule_id: str):
+        """Check if valid schedule."""
+        schedules = {
+            self.schedules[home_id][s]["name"]: self.schedules[home_id][s]["id"]
+            for s in self.schedules.get(home_id, {})
+        }
+        return schedule_id in list(schedules.values())
 
-class HomeStatus:
-    def __init__(self, auth: NetatmoOAuth2, home_id: str):
+
+class HomeData(AbstractHomeData):
+    """Class of Netatmo energy devices."""
+
+    def __init__(self, auth: NetatmoOAuth2) -> None:
+        """Initialize the Netatmo home data.
+
+        Arguments:
+            auth {NetatmoOAuth2} -- Authentication information with a valid access token
+        """
         self.auth = auth
 
-        self.home_id = home_id
-        post_params = {"home_id": self.home_id}
+    def update(self) -> None:
+        """Fetch and process data from API."""
+        resp = self.auth.post_request(url=_GETHOMESDATA_REQ)
+        if resp is None or "body" not in resp:
+            raise NoDevice("No thermostat data returned by Netatmo server")
 
-        resp = self.auth.post_request(url=_GETHOMESTATUS_REQ, params=post_params)
-        if (
-            "errors" in resp
-            or "body" not in resp
-            or "home" not in resp["body"]
-            or ("errors" in resp["body"] and "modules" not in resp["body"]["home"])
-        ):
-            LOG.error("Errors in response: %s", resp)
-            raise NoDevice("No device found, errors in response")
+        self.raw_data = resp["body"].get("homes")
+        if not self.raw_data:
+            raise NoDevice("No thermostat data available")
 
-        self.raw_data = resp["body"]["home"]
-        self.rooms: Dict = {}
-        self.thermostats: Dict = defaultdict(dict)
-        self.valves: Dict = defaultdict(dict)
-        self.relays: Dict = defaultdict(dict)
+        self.process()
 
+    def switch_home_schedule(self, home_id: str, schedule_id: str) -> Any:
+        """Switch the schedule for a give home ID."""
+        if not self.is_valid_schedule(home_id, schedule_id):
+            raise NoSchedule("%s is not a valid schedule id" % schedule_id)
+
+        post_params = {"home_id": home_id, "schedule_id": schedule_id}
+        resp = self.auth.post_request(url=_SWITCHHOMESCHEDULE_REQ, params=post_params)
+        LOG.debug("Response: %s", resp)
+
+
+class AsyncHomeData(AbstractHomeData):
+    """Class of Netatmo energy devices."""
+
+    def __init__(self, auth: AbstractAsyncAuth) -> None:
+        """Initialize the Netatmo home data.
+
+        Arguments:
+            auth {AbstractAsyncAuth} -- Authentication information with a valid access token
+        """
+        self.auth = auth
+
+    async def async_update(self):
+        """Fetch and process data from API."""
+        resp = await self.auth.async_post_request(url=_GETHOMESDATA_REQ)
+        if resp is None or "body" not in resp:
+            raise NoDevice("No thermostat data returned by Netatmo server")
+
+        self.raw_data = resp["body"].get("homes")
+        if not self.raw_data:
+            raise NoDevice("No thermostat data available")
+
+        self.process()
+
+    async def async_switch_home_schedule(self, home_id: str, schedule_id: str) -> Any:
+        """Switch the schedule for a give home ID."""
+        if not self.is_valid_schedule(home_id, schedule_id):
+            raise NoSchedule("%s is not a valid schedule id" % schedule_id)
+
+        post_params = {"home_id": home_id, "schedule_id": schedule_id}
+        resp = await self.auth.async_post_request(
+            url=_SWITCHHOMESCHEDULE_REQ,
+            params=post_params,
+        )
+        LOG.debug("Response: %s", resp)
+
+
+class AbstractHomeStatus(ABC):
+    """Abstract class of the Netatmo home status."""
+
+    raw_data: Dict = defaultdict(dict)
+    rooms: Dict = defaultdict(dict)
+    thermostats: Dict = defaultdict(dict)
+    valves: Dict = defaultdict(dict)
+    relays: Dict = defaultdict(dict)
+
+    def process(self) -> None:
+        """Process data from API."""
         for room in self.raw_data.get("rooms", []):
             self.rooms[room["id"]] = room
 
@@ -156,6 +191,7 @@ class HomeStatus:
                 self.relays[module["id"]] = module
 
     def get_room(self, room_id: str) -> Dict:
+        """Return room data for a given room id."""
         for key, value in self.rooms.items():
             if value["id"] == room_id:
                 return self.rooms[key]
@@ -171,6 +207,7 @@ class HomeStatus:
         raise InvalidRoom("No room with ID %s" % room_id)
 
     def get_relay(self, room_id: str) -> Dict:
+        """Return relay data for a given room id."""
         for key, value in self.relays.items():
             if value["id"] == room_id:
                 return self.relays[key]
@@ -178,6 +215,7 @@ class HomeStatus:
         raise InvalidRoom("No room with ID %s" % room_id)
 
     def get_valve(self, room_id: str) -> Dict:
+        """Return valve data for a given room id."""
         for key, value in self.valves.items():
             if value["id"] == room_id:
                 return self.valves[key]
@@ -197,18 +235,50 @@ class HomeStatus:
         return self.get_room(room_id).get("therm_measured_temperature")
 
     def boiler_status(self, module_id: str) -> Optional[bool]:
+        """Return the status of the boiler status."""
         return self.get_thermostat(module_id).get("boiler_status")
 
+
+class HomeStatus(AbstractHomeStatus):
+    """Class of the Netatmo home status."""
+
+    def __init__(self, auth: NetatmoOAuth2, home_id: str):
+        """Initialize the Netatmo home status.
+
+        Arguments:
+            auth {NetatmoOAuth2} -- Authentication information with a valid access token
+            home_id {str} -- ID for targeted home
+        """
+        self.auth = auth
+
+        self.home_id = home_id
+
+    def update(self) -> None:
+        """Fetch and process data from API."""
+        post_params = {"home_id": self.home_id}
+
+        resp = self.auth.post_request(url=_GETHOMESTATUS_REQ, params=post_params)
+        if (
+            "errors" in resp
+            or "body" not in resp
+            or "home" not in resp["body"]
+            or ("errors" in resp["body"] and "modules" not in resp["body"]["home"])
+        ):
+            LOG.error("Errors in response: %s", resp)
+            raise NoDevice("No device found, errors in response")
+
+        self.raw_data = resp["body"]["home"]
+
+        self.process()
+
     def set_thermmode(
         self,
         mode: str,
         end_time: int = None,
         schedule_id: str = None,
     ) -> Optional[str]:
-        post_params = {
-            "home_id": self.home_id,
-            "mode": mode,
-        }
+        """Set thermotat mode."""
+        post_params = {"home_id": self.home_id, "mode": mode}
         if end_time is not None and mode in ("hg", "away"):
             post_params["endtime"] = str(end_time)
 
@@ -224,11 +294,8 @@ class HomeStatus:
         temp: float = None,
         end_time: int = None,
     ) -> Optional[str]:
-        post_params = {
-            "home_id": self.home_id,
-            "room_id": room_id,
-            "mode": mode,
-        }
+        """Set room themperature set point."""
+        post_params = {"home_id": self.home_id, "room_id": room_id, "mode": mode}
         # Temp and endtime should only be send when mode=='manual', but netatmo api can
         # handle that even when mode == 'home' and these settings don't make sense
         if temp is not None:
@@ -238,3 +305,80 @@ class HomeStatus:
             post_params["endtime"] = str(end_time)
 
         return self.auth.post_request(url=_SETROOMTHERMPOINT_REQ, params=post_params)
+
+
+class AsyncHomeStatus(AbstractHomeStatus):
+    """Class of the Netatmo home status."""
+
+    def __init__(self, auth: AbstractAsyncAuth, home_id: str):
+        """Initialize the Netatmo home status.
+
+        Arguments:
+            auth {AbstractAsyncAuth} -- Authentication information with a valid access token
+            home_id {str} -- ID for targeted home
+        """
+        self.auth = auth
+
+        self.home_id = home_id
+
+    async def async_update(self) -> None:
+        """Fetch and process data from API."""
+        post_params = {"home_id": self.home_id}
+
+        resp = await self.auth.async_post_request(
+            url=_GETHOMESTATUS_REQ,
+            params=post_params,
+        )
+        if (
+            "errors" in resp
+            or "body" not in resp
+            or "home" not in resp["body"]
+            or ("errors" in resp["body"] and "modules" not in resp["body"]["home"])
+        ):
+            LOG.error("Errors in response: %s", resp)
+            raise NoDevice("No device found, errors in response")
+
+        self.raw_data = resp["body"]["home"]
+
+        self.process()
+
+    async def async_set_thermmode(
+        self,
+        mode: str,
+        end_time: int = None,
+        schedule_id: str = None,
+    ) -> Optional[str]:
+        """Set thermotat mode."""
+        post_params = {"home_id": self.home_id, "mode": mode}
+        if end_time is not None and mode in ("hg", "away"):
+            post_params["endtime"] = str(end_time)
+
+        if schedule_id is not None and mode == "schedule":
+            post_params["schedule_id"] = schedule_id
+
+        return await self.auth.async_post_request(
+            url=_SETTHERMMODE_REQ,
+            params=post_params,
+        )
+
+    async def async_set_room_thermpoint(
+        self,
+        room_id: str,
+        mode: str,
+        temp: float = None,
+        end_time: int = None,
+    ) -> Optional[str]:
+        """Set room themperature set point."""
+        post_params = {"home_id": self.home_id, "room_id": room_id, "mode": mode}
+        # Temp and endtime should only be send when mode=='manual', but netatmo api can
+        # handle that even when mode == 'home' and these settings don't make sense
+        if temp is not None:
+            post_params["temp"] = str(temp)
+
+        if end_time is not None:
+            post_params["endtime"] = str(end_time)
+
+        return await self.auth.async_post_request(
+            url=_SETROOMTHERMPOINT_REQ,
+            params=post_params,
+        )
